/*  This file is part of the KDE project
    Copyright (C) 2002 Matthias Kretz <kretz@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2
    as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/* $Id$ */

#ifndef __kviewpresenter_h
#define __kviewpresenter_h

#include <tdeparts/plugin.h>
#include <kurl.h>

#include <tqvaluelist.h>

namespace KImageViewer { class Viewer; }

class ImageListDialog;
class ImageListItem;
class TQListViewItem;
class TQTimer;
class TDEToggleAction;
class TDEAction;

class KViewPresenter : public KParts::Plugin
{
	TQ_OBJECT
  
public:
	KViewPresenter( TQObject* parent, const char* name, const TQStringList & );
	virtual ~KViewPresenter();

protected:
	struct ImageInfo
	{
		KURL url;

		ImageInfo()
			: url()
		{
		}
		ImageInfo( const KURL & url )
			: url( url )
		{
		}
		bool operator==( const ImageInfo & i1 )
		{
			return url.prettyURL() == i1.url.prettyURL();
		}
		bool operator!=( const ImageInfo & i1 )
		{
			return url.prettyURL() != i1.url.prettyURL();
		}
		bool operator>( const ImageInfo & i1 )
		{
			return url.prettyURL() > i1.url.prettyURL();
		}
		bool operator<( const ImageInfo & i1 )
		{
			return url.prettyURL() < i1.url.prettyURL();
		}
	};
	bool eventFilter( TQObject *, TQEvent * );

private slots:
	void slotImageOpened( const KURL & );
	void slotImageList();
	void slotOpenFiles();
	void slotClose();
	void changeItem( TQListViewItem * );
	void prev();
	void next();
	void slideshow( bool );
	void setSlideshowInterval( int );
	void shuffle();
	void closeAll();
	void loadList();
	void saveList();

private:
	void makeCurrent( TQListViewItem * );

	KImageViewer::Viewer * m_pViewer;
	ImageListDialog * m_pImageList;
	TDEToggleAction * m_paSlideshow;
	TDEAction * m_paFileOpen;
	TDEAction * m_paFileClose;

	TQValueList<ImageInfo> m_imagelist;
	bool m_bDontAdd;
	ImageListItem * m_pCurrentItem;
	TQTimer * m_pSlideshowTimer;
};
#endif
